#ifndef _EVGENERATOR_H
#define _EVGENERATOR_H

#include "string.h"
#include <time.h>

#include <TLorentzVector.h>

#include "../generalUtils/constants.h"
#include "../generalUtils/kinematics.h"
#include "sigmaKY.h"




class evGenerator {

  string type;
  double m1;
  double m2;  
  double Ebeam;
  double Q2min;
  double Q2max;
  double Wmin;
  double Wmax;
  double d5sigmaMax;
  sigmaKY  *model;
  
  int nEvent;

public:

evGenerator(string t, double E, 
            double q2min, double q2max,
	    double wmin,  double wmax)
  {
  type = t;
  Ebeam = E;
  Q2min = q2min;      
  Q2max = q2max;
  Wmin = wmin;
  Wmax = wmax;
  nEvent = 0;

    
  if(type == "KLambda") {
    m1 = massKaon; 
    m2 = massLambda;
  } else if(type == "KSigma"){
    m1 = massKaon; 
    m2 = massSigma0;
  } else {
    cerr << " Error! evGenerator::evGenerator Wrong reaction type.";
    cerr << " It must be KLambda or Ksigma";  
    exit(1);
  }  
  
  // read data file
  model = new sigmaKY(type);
  model->use_costeta();
  
  // Find maximum of the cross section
  d5sigmaMax = model->d5sigma_max(Ebeam, Q2min, Q2max, Wmin, Wmax);
	  
  // initialize random seed:
  srand (time(NULL));
 
};

~evGenerator(){
  if(type == "KL07" || type == "KL11" || type == "KS") {
    delete model;
  }
};




		 
		 
// all parameters are output parameters
// 4-momenta of particles in the final state in LAB frame:
//    Pefin, PK, PL -> electron, Kaon, Lambda/Sigma
// 4 momenta of 

void getEvent(double &Q2, double &W, 
              TLorentzVector &Pefin, TLorentzVector &PK, TLorentzVector &PY,
	      TLorentzVector &Ppfin, TLorentzVector &Ppim, TLorentzVector &Pgam) {

  int nTry=0;
  nEvent=0;
  while(true) {
 
    Q2 = randomIntv(Q2min, Q2max);
    W  = randomIntv(Wmin, Wmax);
    double cosThetaK = randomIntv(-0.999999,0.999999);
    double thetaK = acos(cosThetaK);
    double phiK = randomIntv(0.0, 2.*constantPi);

    double d5sigma;
    d5sigma = model->d5sigma(Ebeam, Q2, W, thetaK, phiK);
        
    nTry++;
        
    if(randomIntv(0.,1.) < d5sigma/d5sigmaMax) {
       nEvent++;
       
       //cout << "   NNN  " << nTry << " " << nEvent << endl;

            
       double W2 = W*W;
       double omega = getomega(Q2,W);
       if(omega <=0. ) continue;
       double Ee = Ebeam - omega;
       if(Ee<massElectron) continue;
       double arg = 1. - Q2/(2.*Ebeam*Ee);
       if( fabs(arg) > 1. ) continue;
       double theta = acos(arg);
       double phi = randomIntv(0.0, 2.*constantPi);
       double gamma = getGamma(Ebeam,Q2,W);
       
       // 4-momentum of final electron in LAB frame
       Pefin.SetXYZT(1.,1.,1.,Ee);
       Pefin.SetRho(sqrt(Ee*Ee - massElectron2)); 
       Pefin.SetTheta(theta);
       Pefin.SetPhi(phi); 
     
       cms2lab(W, Q2, phi, Ebeam, thetaK, phiK, m1, m2, PK, PY);     


       // Decay of Lambda into proton and pi minus. 
       if(type == "KLambda" ) {
	 getLdecayProd(PY, Ppfin, Ppim);
       } else {
       // Decay of Sigma into proton and pi minus and gamma. 
         TLorentzVector PL;
         getSdecayProd(PY, PL, Ppfin, Ppim, Pgam);
       }
       
       //cout << " PY " << PY.E() <<" "<< PY.Px() <<" "<< PY.Py() <<" "<< PY.Pz() << endl; 
       //cout << " PS " << (Ppfin+Ppim).E() <<" "<< (Ppfin+Ppim).Px() <<" "<< (Ppfin+Ppim).Py() <<" "<< (Ppfin+Ppim).Pz() << endl; 
       //cout << " PK " << (PK).E() <<" "<< (PK).Px() <<" "<< (PK).Py() <<" "<< (PK).Pz() << endl; 
       //cout << " Pp " << (Ppfin).E() <<" "<< (Ppfin).Px() <<" "<< (Ppfin).Py() <<" "<< (Ppfin).Pz() << endl; 
       //cout << " Pi " << (Ppim).E()  <<" "<< (Ppim).Px()  <<" "<< (Ppim).Py()  <<" "<< (Ppim).Pz()  << endl; 
       
       return;
    
    }

  
  }

}; //end getEvent(...)

	
};






#endif 
