#ifndef _SIGMAKY_H
#define _SIGMAKY_H

#include "string.h"

#include "../generalUtils/utils.h"
#include "../generalUtils/constants.h"
#include "../generalUtils/kinematics.h"


class sigmaKY {
	int varsnum;
	int obsnum;
	char vars[7][20];
	char vardims[7][20];
	int count_Q2;
	int count_W;
	int count_cos_t;
	char eps_or_Eb;
	char teta_or_cost;
	char rads_or_degrees;
	double beam_energy;
	double step_Q2;
	double step_W;
	double step_cos_t;
	double Q2[61];	
	double W[35];	
	double cos_t[21];
	double L[61][35][21];
	double T[61][35][21];
	double LT[61][35][21];
	double TT[61][35][21];

	double dsigmaXX_dphi(double XX[61][35][21], int iQ2, double _Q2, int iW, double _W);					//complete
	double dsigmaXX_domega(double XX[61][35][21], int iQ2, double _Q2, int iW, double _W, int icos_t, double _cos_t);					//complete
	int srch_Q2(double value);
	int srch_W(double value);
	int srch_cos_t(double value);
	double fxxx(int i, int j, int k, double eps, double phi);
	double fxxx(int i, int j, int k, double eps);
public:
	sigmaKY();
	sigmaKY(string path, string, double, double, double);
	//~sigmaKY(){ delete hres; }

			//switches
	void use_beam_energy()  {eps_or_Eb = 1;}
	void use_epsilon()	{eps_or_Eb = 0;}
	void use_theta()	{teta_or_cost = 1;}
	void use_costeta()	{teta_or_cost = 0;}
	void use_radians()	{rads_or_degrees = 1;}
	void use_degrees()	{rads_or_degrees = 0;}
	double StepW() {return step_W;}
	double StepQ2() {return step_Q2;}
	double MinW() {return W[0];}
	double MinQ2() {return Q2[0];}
	double MaxW() {return W[count_W-1];}
	double MaxQ2() {return Q2[count_Q2-1];}
	double BeamEnergy() {return beam_energy;}
	int get_count_cos_t()	{return count_cos_t;}
	double *get_cos_t() {return cos_t;}
	void setBeamEnergy(double _beam_energy) {beam_energy = _beam_energy;}

        double d2sigma(double Ebeam, double Q2, double W, 
	               double thetaK, double phiK);

	double d5sigma(double beam_energy, double _Q2, double _W, 
	               double thetaK, double phiK);

        double d5sigma_max(double Ebeam, double Q2min, double Q2max,
                           double Wmin,  double Wmax  );

	double dsigma_dcos(double _beam_energy, double _Q2, double _W, double teta);					//complete

        double sigma_int(double Ebeam, double Q2, double W);

	double *dsigma_dcos(double _beam_energy, double _Q2, double _W);								//complete

	double dsigma_dphi(double _beam_enegry, double _Q2, double _W, double phi);					//complete but perhaps incorrectly

	double *dsigma_dphi(double _beam_enegry, double _Q2, double _W);								//complete

	double dsigma_domega(double _beam_energy, double _Q2, double _W, double teta, double phi);	//complete

	double *dsigma_domega(double _beam_energy, double _Q2, double _W, double teta);				//complete

	double sigma(double _beam_energy, double _Q2, double _W);

	double **sigma_grid(double _beam_energy);
};





sigmaKY::sigmaKY(string path, string addr="", double a12=0., double a32=0., double s12=0.)
{


        ifstream input;
        if(path=="KLambda") {
	  input.open("./data/work_K+Lambda0_strange_calc_RPR-2011.txt");
	} else if(path=="KSigma") {
	  input.open("./data/work_K+Sigma0_strange_calc_RPR-2007.txt");
	} else {
	  input.open(path.c_str());
	}
	

	this->use_beam_energy();
	this->use_costeta();
	this->use_radians();

	varsnum = 7;
	obsnum = 44835;
	count_Q2 = 61;
	count_W = 35;
	count_cos_t = 21;


	if (!input)	{
		std::cout << "Unable to open file " << path << '\n';
		exit(1);
	}
	
	
	double _Q2, _W, _cos_t;
	char junk[20];
	for(int i = 0; i < varsnum; i++) { input >> vars[i]; }
	for(int i = 0; i < varsnum; i++) input >> vardims[i];
	for(int i = 0; i < varsnum; i++) input >> junk;
	
	for(int i = 0; i < count_Q2; i++)	{
		for(int j = 0; j < count_W; j++)	{
			for(int k = 0; k < count_cos_t; k++)	{
				input >> _Q2 >> _W >> _cos_t >> L[i][j][k] >> T[i][j][k] >> LT[i][j][k] >> TT[i][j][k];
				
				//cout <<" LLL "<< L[i][j][k] << T[i][j][k] << LT[i][j][k] << TT[i][j][k] << endl;
				
				//if(addr=="R"){ 
				//  double sigt, sigl;
				//  hres->getSigtSigL(_Q2, _W, _cos_t, sigt, sigl);
				//  L[i][j][k] += sigt;
				//  T[i][j][k] += sigl;
				//  //cout << " LL " << L[i][j][k] <<" "<< sigt <<  endl;
				//}
				
				if (i == 0 && j ==0) cos_t[k] = _cos_t;
				
			}
			if(i == 0) W[j] = _W;
		
		}
		
		
		Q2[i] = _Q2;
	}
	input.close();

	step_Q2 = (Q2[count_Q2-1] - Q2[0]) / (double)(count_Q2-1);
	step_W = (W[count_W-1] - W[0]) / (double)(count_W-1);
	step_cos_t = (cos_t[count_cos_t-1] - cos_t[0]) / (double)(count_cos_t-1);
		
	
}



int sigmaKY::srch_Q2(double value)
{
	if (value <= Q2[count_Q2-1] && value >= Q2[0])	{
		int pre_index = (int)((value - Q2[0]) / step_Q2);
		if (Q2[pre_index] <= value && Q2[pre_index+1] > value) return pre_index;
			else if (Q2[pre_index] > value) return pre_index-1;
				else if (Q2[pre_index+1] <= value) return pre_index+1;
	} else {
		std::cout << "Select the value of Q^2 in the interval [" << Q2[0] << ", " << Q2[count_Q2-1] << "] GeV^2\n";
		exit(1);
	}
}
int sigmaKY::srch_W(double value)
{
        if(value<W[0]) return -1;
	if (value <= W[count_W-1] && value >= W[0])	{
		int pre_index = (int)((value - W[0]) / step_W);
				
		if (W[pre_index] <= value && W[pre_index+1] > value) return pre_index;
			else if (W[pre_index] > value) return pre_index-1;
				else if (W[pre_index+1] <= value) return pre_index+1;
	} else {
		std::cout << "Select the value of W in the interval [" << W[0] << ", " << W[count_W-1] << "] GeV\n";
		exit(1);
	}
}
int sigmaKY::srch_cos_t(double value)
{
	if (value <= cos_t[count_cos_t-1] && value >= cos_t[0])	{
		int pre_index = (int)((value - cos_t[0]) / step_cos_t);
		
		// std::cout << " SS1 " << value <<" "<< cos_t[0]<<" "<< step_cos_t <<" "<< pre_index << std::endl;

		
		if (cos_t[pre_index] <= value && cos_t[pre_index+1] > value) return pre_index;
			else if (cos_t[pre_index] > value) return pre_index-1;
				else if (cos_t[pre_index+1] <= value) return pre_index+1;
	} else {
		std::cout << "Select the value of cos(teta) in the interval [-1, 1]\n";
		exit(1);
	}
}


double sigmaKY::fxxx(int i, int j, int k, double eps, double phi)	{
	return T[i][j][k] + eps * L[i][j][k] + eps * TT[i][j][k] * cos(2*phi) + sqrt(eps*(1+eps)) * LT[i][j][k] * cos(phi);
}
double sigmaKY::fxxx(int i, int j, int k, double eps)	{
	return T[i][j][k] + eps * L[i][j][k];
}
double sigmaKY::dsigma_domega(double _beam_energy, double _Q2, double _W, double teta, double phi)
{
	double _cos_t;
	if (teta_or_cost) _cos_t = cos(teta);
		else _cos_t = teta;
		
	int iQ2 = this->srch_Q2(_Q2);
	int iW = this->srch_W(_W);
	if(iW==-1) return 0.;
	int icos_t = this->srch_cos_t(_cos_t);
	double nu = (_Q2 + _W*_W - massProton2) / (2 * massProton);
	double eps;
	if (eps_or_Eb) eps = 1 / (1 + 2*((_Q2 + nu*nu) / (4*(_beam_energy - nu)*_beam_energy - _Q2)));
		else eps = _beam_energy;


	double _const = step_Q2 * step_W * step_cos_t;
	double interpolated_value = 0;


	interpolated_value += this->fxxx(iQ2,iW,icos_t,eps,phi)/_const * (Q2[iQ2+1] - _Q2) * (W[iW+1] - _W) * (cos_t[icos_t+1] - _cos_t);
	interpolated_value += this->fxxx(iQ2,iW,icos_t+1,eps,phi)/_const * (Q2[iQ2+1] - _Q2) * (W[iW+1] - _W) * (_cos_t - cos_t[icos_t]);
	interpolated_value += this->fxxx(iQ2,iW+1,icos_t,eps,phi)/_const * (Q2[iQ2+1] - _Q2) * (_W - W[iW]) * (cos_t[icos_t+1] - _cos_t);
	interpolated_value += this->fxxx(iQ2,iW+1,icos_t+1,eps,phi)/_const * (Q2[iQ2+1] - _Q2) * (_W - W[iW]) * (_cos_t - cos_t[icos_t]);
	interpolated_value += this->fxxx(iQ2+1,iW,icos_t,eps,phi)/_const * (_Q2 - Q2[iQ2]) * (W[iW+1] - _W) * (cos_t[icos_t+1] - _cos_t);
	interpolated_value += this->fxxx(iQ2+1,iW,icos_t+1,eps,phi)/_const * (_Q2 - Q2[iQ2]) * (W[iW+1] - _W) * (_cos_t - cos_t[icos_t]);
	interpolated_value += this->fxxx(iQ2+1,iW+1,icos_t,eps,phi)/_const * (_Q2 - Q2[iQ2]) * (_W - W[iW]) * (cos_t[icos_t+1] - _cos_t);
	interpolated_value += this->fxxx(iQ2+1,iW+1,icos_t+1,eps,phi)/_const * (_Q2 - Q2[iQ2]) * (_W - W[iW]) * (_cos_t - cos_t[icos_t]);

	return interpolated_value;
}








double sigmaKY::d2sigma(double Ebeam, double Q2, double W, double thetaK, double phiK)
{

return this->dsigma_domega(Ebeam, Q2, W, cos(thetaK), phiK);

}






double sigmaKY::d5sigma(double Ebeam, double Q2, double W, double thetaK, double phiK)
{
  double mp=massProton;
  double mp2 = mp*mp;
  double pi=constantPi;
  double alpha=constantAlpha;
  double W2 = W*W;
  
  double omega = (W2 + Q2 -mp2)/(2.*mp);
  if(omega<0. || omega>Ebeam) return 0.;
  double eps2 = Ebeam - omega;
  double k2 = Q2 + omega*omega;
  if(k2<0) return 0.;
  double arg = 1. - Q2/(2.*Ebeam*eps2);
  if(arg>1. || arg<-1.) return 0.;
  double theta = 2.*acos(arg);
  double epsilon = 1./( 1 + (2.*k2/Q2)*tan(theta/2.) );
  double gamma = getGamma(Ebeam, Q2, W);
    
  double d5sig = gamma *
                 this->dsigma_domega(Ebeam, Q2, W, cos(thetaK), phiK);
  return d5sig;

}



double sigmaKY::d5sigma_max(double Ebeam, double Q2min, double Q2max,
                                       double Wmin,  double Wmax  )
{

        double pi=constantPi;

	// Find maximum of the cross section
        int nQ2=100;
        int nW=20;
        int nCosThetaK=20;
        int nPhiK=20;
	double d5sigmaMax=0.;
        for(int iQ2=0; iQ2<nQ2; iQ2++) {
	double Q2 = Q2min + (Q2max-Q2min)*iQ2/(nQ2-1);
        for(int iW=0;   iW<nW;  iW++) {
	double W =  Wmin +  (Wmax-Wmin)*iW/(nW-1);
	for(int iCosThK=0; iCosThK<20; iCosThK++) {
	double cosThetaK = -0.9999 + (0.9999-(-0.9999))*iCosThK/(nCosThetaK-1);
	double thetaK = acos(cosThetaK);
        for(int iPhiK=0; iPhiK<nPhiK;  iPhiK++) {
	double phiK = 0. + (2.*pi-0.)*iPhiK/(nPhiK-1);
          double d5sig = d5sigma(Ebeam, Q2, W, thetaK, phiK);
          if(d5sig>d5sigmaMax) d5sigmaMax=d5sig;
    
	}
        }
	}
        }
        return d5sigmaMax;

}








double *sigmaKY::dsigma_dcos(double _beam_energy, double _Q2, double _W)
{
	int iQ2 = this->srch_Q2(_Q2);
	int iW = this->srch_W(_W);
	double nu = (_Q2 + _W*_W - massProton2) / (2 * massProton);
	double eps;
	if (eps_or_Eb) eps = 1 / (1 + 2*((_Q2 + nu*nu) / (4*(_beam_energy - nu)*_beam_energy - _Q2)));
		else eps = _beam_energy;

	double _const = step_Q2 * step_W;
	double *distr = (double*) calloc(count_cos_t, sizeof(double));
	for(int i = 0; i < count_cos_t; i++)	{
		distr[i] += this->fxxx(iQ2, iW, i, eps) / _const * (Q2[iQ2+1] - _Q2) * (W[iW+1] - _W);
		distr[i] += this->fxxx(iQ2, iW+1, i, eps) / _const * (Q2[iQ2+1] - _Q2) * (_W - W[iW]);
		distr[i] += this->fxxx(iQ2+1, iW, i, eps) / _const * (_Q2 - Q2[iQ2]) * (W[iW+1] - _W);
		distr[i] += this->fxxx(iQ2+1, iW+1, i, eps) / _const * (_Q2 - Q2[iQ2]) * (_W - W[iW]);
		distr[i] = 2 * M_PI * distr[i];
	}
		
	return distr;
}

double sigmaKY::dsigma_dcos(double _beam_energy, double _Q2, double _W, double teta)
{
	double _cos_t;
	if (teta_or_cost) _cos_t = cos(teta);
        	else _cos_t = teta;
	int iQ2 = this->srch_Q2(_Q2);
	int iW = this->srch_W(_W);
	int icos_t = this->srch_cos_t(_cos_t);
	if(iW==-1) return 0.;
	double nu = (_Q2 + _W*_W - massProton2) / (2 * massProton);
	double eps;
	if (eps_or_Eb) eps = 1 / (1 + 2*((_Q2 + nu*nu) / (4*(_beam_energy - nu)*_beam_energy - _Q2)));
		else eps = _beam_energy;
	double _const = (Q2[iQ2+1] - Q2[iQ2]) * (W[iW+1] - W[iW]) * (cos_t[icos_t+1] - cos_t[icos_t]);
	double interpolated_value = 0;

        //std::cout << " QQWW " << _Q2 << " "<< iQ2 << " " << _W << " "<<iW <<" "<< teta << " "<< icos_t << std::endl;


	interpolated_value += this->fxxx(iQ2,iW,icos_t,eps)/_const * (Q2[iQ2+1] - _Q2) * (W[iW+1] - _W) * (cos_t[icos_t+1] - _cos_t);
	interpolated_value += this->fxxx(iQ2,iW,icos_t+1,eps)/_const * (Q2[iQ2+1] - _Q2) * (W[iW+1] - _W) * (_cos_t - cos_t[icos_t]);
	interpolated_value += this->fxxx(iQ2,iW+1,icos_t,eps)/_const * (Q2[iQ2+1] - _Q2) * (_W - W[iW]) * (cos_t[icos_t+1] - _cos_t);
	interpolated_value += this->fxxx(iQ2,iW+1,icos_t+1,eps)/_const * (Q2[iQ2+1] - _Q2) * (_W - W[iW]) * (_cos_t - cos_t[icos_t]);
	interpolated_value += this->fxxx(iQ2+1,iW,icos_t,eps)/_const * (_Q2 - Q2[iQ2]) * (W[iW+1] - _W) * (cos_t[icos_t+1] - _cos_t);
	interpolated_value += this->fxxx(iQ2+1,iW,icos_t+1,eps)/_const * (_Q2 - Q2[iQ2]) * (W[iW+1] - _W) * (_cos_t - cos_t[icos_t]);
	interpolated_value += this->fxxx(iQ2+1,iW+1,icos_t,eps)/_const * (_Q2 - Q2[iQ2]) * (_W - W[iW]) * (cos_t[icos_t+1] - _cos_t);
	interpolated_value += this->fxxx(iQ2+1,iW+1,icos_t+1,eps)/_const * (_Q2 - Q2[iQ2]) * (_W - W[iW]) * (_cos_t - cos_t[icos_t]);

	//cout << "PPP " << _Q2 << " "<< _W <<" "<< _cos_t << endl;
        //double sigt=0, sigl=0, sig;
	
	//hres->getSigtSigL(_Q2, _W, _cos_t, sigt, sigl);
		
        //sig= hres->dsigma_dcos(11., _Q2, _W, teta);
	//cout << " EEE " << sig << " " << endl;

	return 2 * M_PI * interpolated_value; // * (389.5*(sigt + eps*sigl)); // * hres->dsigma_dcos(getEbeamEps(eps, _Q2, _W), _Q2, _W, teta);
}


double sigmaKY::sigma_int(double Ebeam, double Q2, double W)
{
   double sig = 0.;
   double dC=2./20.;
   for(int iC=0; iC<20; iC++) {
     double c = -1. + 0.05 + iC*dC;
     
     //cout << "DCO " << dsigma_dcos(Ebeam, Q2, W, c)<< endl;
     
     sig += dsigma_dcos(Ebeam, Q2, W, c)*dC; 
   }
   return sig;   
}



double sigmaKY::dsigmaXX_dphi(double XX[61][35][21], int iQ2, double _Q2, int iW, double _W)
{
	double _const = (Q2[iQ2+1] - Q2[iQ2]) * (W[iW+1] - W[iW]);
	double dsigmaXX_prev = 0, dsigmaXX = 0, result = 0;

	dsigmaXX_prev += XX[iQ2][iW][0] / _const * (Q2[iQ2+1] - _Q2) * (W[iW+1] - _W);
	dsigmaXX_prev += XX[iQ2][iW+1][0] / _const * (Q2[iQ2+1] - _Q2) * (_W - W[iW]);
	dsigmaXX_prev += XX[iQ2+1][iW][0] / _const * (_Q2 - Q2[iQ2]) * (W[iW+1] - _W);
	dsigmaXX_prev += XX[iQ2+1][iW+1][0] / _const * (_Q2 - Q2[iQ2]) * (_W - W[iW]);
	for (int i = 0; i < count_cos_t-1; i++)	{
		dsigmaXX += XX[iQ2][iW][i+1] / _const * (Q2[iQ2+1] - _Q2) * (W[iW+1] - _W);
		dsigmaXX += XX[iQ2][iW+1][i+1] / _const * (Q2[iQ2+1] - _Q2) * (_W - W[iW]);
		dsigmaXX += XX[iQ2+1][iW][i+1] / _const * (_Q2 - Q2[iQ2]) * (W[iW+1] - _W);
		dsigmaXX += XX[iQ2+1][iW+1][i+1] / _const * (_Q2 - Q2[iQ2]) * (_W - W[iW]);

		result += (dsigmaXX_prev + dsigmaXX) / 2 * step_cos_t;

		dsigmaXX_prev = dsigmaXX;
		dsigmaXX = 0;
	}
	return result;
}
double *sigmaKY::dsigma_domega(double _beam_energy, double _Q2, double _W, double teta)
{
	double _cos_t;
	if (teta_or_cost) _cos_t = cos(teta);
		else _cos_t = teta;
	int iQ2 = this->srch_Q2(_Q2);
	int iW = this->srch_W(_W);
	int icos_t = this->srch_cos_t(_cos_t);
	double nu = (_Q2 + _W*_W - massProton2) / (2 * massProton);
	double eps;
	if (eps_or_Eb) eps = 1 / (1 + 2*((_Q2 + nu*nu) / (4*(_beam_energy - nu)*_beam_energy - _Q2)));
       else eps = _beam_energy;

	double *distr = (double*) calloc(4, sizeof(double));

	distr[0] = this->dsigmaXX_domega(T, iQ2, _Q2, iW, _W, icos_t, _cos_t);
	distr[1] = this->dsigmaXX_domega(L, iQ2, _Q2, iW, _W, icos_t, _cos_t) * eps;
	distr[2] = this->dsigmaXX_domega(TT, iQ2, _Q2, iW, _W, icos_t, _cos_t) * eps;
	distr[3] = this->dsigmaXX_domega(LT, iQ2, _Q2, iW, _W, icos_t, _cos_t) * sqrt(eps*(1+eps));
	return distr;
}
double sigmaKY::dsigmaXX_domega(double XX[61][35][21], int iQ2, double _Q2, int iW, double _W, int icos_t, double _cos_t)
{
	double _const = step_Q2 * step_W * step_cos_t;
	double interpolated_value = 0;
	interpolated_value += XX[iQ2][iW][icos_t]/_const * (Q2[iQ2+1] - _Q2) * (W[iW+1] - _W) * (cos_t[icos_t+1] - _cos_t);
	interpolated_value += XX[iQ2][iW][icos_t+1]/_const * (Q2[iQ2+1] - _Q2) * (W[iW+1] - _W) * (_cos_t - cos_t[icos_t]);
	interpolated_value += XX[iQ2][iW+1][icos_t]/_const * (Q2[iQ2+1] - _Q2) * (_W - W[iW]) * (cos_t[icos_t+1] - _cos_t);
	interpolated_value += XX[iQ2][iW+1][icos_t+1]/_const * (Q2[iQ2+1] - _Q2) * (_W - W[iW]) * (_cos_t - cos_t[icos_t]);
	interpolated_value += XX[iQ2+1][iW][icos_t]/_const * (_Q2 - Q2[iQ2]) * (W[iW+1] - _W) * (cos_t[icos_t+1] - _cos_t);
	interpolated_value += XX[iQ2+1][iW][icos_t+1]/_const * (_Q2 - Q2[iQ2]) * (W[iW+1] - _W) * (_cos_t - cos_t[icos_t]);
	interpolated_value += XX[iQ2+1][iW+1][icos_t]/_const * (_Q2 - Q2[iQ2]) * (_W - W[iW]) * (cos_t[icos_t+1] - _cos_t);
	interpolated_value += XX[iQ2+1][iW+1][icos_t+1]/_const * (_Q2 - Q2[iQ2]) * (_W - W[iW]) * (_cos_t - cos_t[icos_t]);
	std::cout << interpolated_value << std::endl;
	return interpolated_value;
}
double sigmaKY::dsigma_dphi(double beam_enegry, double _Q2, double _W, double phi)
{
	int iQ2 = this->srch_Q2(_Q2);
	int iW = this->srch_W(_W);
	if(iW==-1) return 0.;
	double nu = (_Q2 + _W*_W - massProton2) / (2 * massProton);
	double eps;
	
	if (eps_or_Eb) { eps = 1 / (1 + 2*((_Q2 + nu*nu) / (4*(beam_enegry - nu)*beam_enegry - _Q2))); }
	else           { eps = beam_enegry; }

	double *distr = (double*) calloc(4, sizeof(double));
	distr[0] = this->dsigmaXX_dphi(T, iQ2, _Q2, iW, _W);
	distr[1] = this->dsigmaXX_dphi(L, iQ2, _Q2, iW, _W) * eps;
	distr[2] = this->dsigmaXX_dphi(TT, iQ2, _Q2, iW, _W) * eps;
	distr[3] = this->dsigmaXX_dphi(LT, iQ2, _Q2, iW, _W) * sqrt(eps*(1+eps));
	return distr[0] + distr[1] + distr[2]*cos(2*phi) + distr[3]*cos(phi);
}

double *sigmaKY::dsigma_dphi(double _beam_energy, double _Q2, double _W)
{
	int iQ2 = this->srch_Q2(_Q2);
	int iW = this->srch_W(_W);
	double nu = (_Q2 + _W*_W - massProton2) / (2 * massProton);
	double eps;
	if (eps_or_Eb) eps = 1 / (1 + 2*((_Q2 + nu*nu) / (4*(_beam_energy - nu)*_beam_energy - _Q2)));
		else eps = _beam_energy;

	double *distr = (double*) calloc(4, sizeof(double));
	distr[0] = this->dsigmaXX_dphi(T, iQ2, _Q2, iW, _W);
	distr[1] = this->dsigmaXX_dphi(L, iQ2, _Q2, iW, _W) * eps;
	distr[2] = this->dsigmaXX_dphi(TT, iQ2, _Q2, iW, _W) * eps;
	distr[3] = this->dsigmaXX_dphi(LT, iQ2, _Q2, iW, _W) * sqrt(eps*(1+eps));
	return distr;
}

double sigmaKY::sigma(double _beam_energy, double _Q2, double _W)
{
	double *distr = this->dsigma_dphi(_beam_energy, _Q2, _W);
	return 2 * M_PI * (distr[0]+ distr[1]);
}
double **sigmaKY::sigma_grid(double _beam_energy) /*ONLY ENERGY*//*ONLY ENERGY*//*ONLY ENERGY*//*ONLY ENERGY*//*ONLY ENERGY*/
{
	double nu, eps, buff1, buff2;

	double **sigma = new double *[61];
		

	for(int i = 0; i < 61; i++)	{
		sigma[i] = new double [35];
		for(int j = 0; j < 35; j++) {
			nu = (Q2[i] + W[j]*W[j] - massProton2) / (2 * massProton);
			eps = 1 / (1 + 2*((Q2[i] + nu*nu) / (4*(_beam_energy - nu)*_beam_energy - Q2[i])));
			buff1 = 0;
			buff2 = 0;
			for(int k = 0; k < 21; k++) {
				buff1 += T[i][j][k];
				buff2 += L[i][j][k];
			}
			sigma[i][j] = buff1 - T[i][j][0] - T[i][j][20] + (buff2 - (L[i][j][0] + L[i][j][20]) / 2.) * eps;
		}
	}
	return sigma;
}




#endif 
